#include <fstream>

#include "CBinaire.h"
#include "CChainePersistante.h"
#include "CParcellePersistante.h"

/*! \mainpage Classes génériques pour la gestion de la persistance en C++
 * \section Diagramme Diagramme de classes
 * \diafile persistance.dia
 *
 * Ce diagramme comporte : \n
 * - en jaune pâle : une classe CBasePersistant assurant la gestion primaire, \n
 * - en jaune pâle : une classe abstraite CPersistant basée sur une stratégie (template) base des classes persistantes à construire, \n
 * - en jaune : une classe exemple CChainePersistante basée sur une stratégie (template) \n
 * Cette classe n'utilise pas les métadonnées pour lire et écrire dans un flux. \n
 * Le programme de test défini une CChainePersistante basée sur une stratégie de stockage binaire CBinaire \n
 * - en jaune : une classe exemple CParcellePersistante basée sur une stratégie (template) \n
 * Cette classe utilise les métadonnées pour lire et écrire dans un flux. \n
 * Le programme de test défini une CParcellePersistante basée sur une stratégie de stockage binaire CBinaire \n
 * - en jaune : une classe CBinaire pour un exemple de stratégie de stockage binaire.
 */
 // ________________________________________________________________________________________
int main () {

	persistance::CChainePersistante<persistance::CBinaire>::referencer ();
	persistance::CParcellePersistante<persistance::CBinaire>::referencer ();

	persistance::CChainePersistante<persistance::CBinaire> lvChaineP1 ("TEST CHAINE PERSISTANTE", true, false);
	persistance::CParcellePersistante<persistance::CBinaire> lvParcelleAB100 ("07003000AB0100", +3695., true, false);
	persistance::CParcellePersistante<persistance::CBinaire> lvParcelleAB65 ("07003000AB0065", +21105., true, false);

	std::fstream* laFlux = new std::fstream ("/home/jcc/persistant.data", persistance::CBinaire::mode () | std::ios_base::in | std::ios_base::out);
	if (laFlux->fail ()) {
		delete laFlux;
		delete new std::ofstream ("/home/jcc/persistant.data");
		laFlux = new std::fstream ("/home/jcc/persistant.data", persistance::CBinaire::mode () | std::ios_base::in | std::ios_base::out);
		};

	persistance::CPersistant<persistance::CBinaire>::toutEcrire (*laFlux);
	persistance::CPersistant<persistance::CBinaire>::lire (*laFlux);

	delete laFlux;
	return 0;
	};
