#include <algorithm>

#include "CBasePersistant.h"

namespace persistance {

// ________________________________________________________________________________________
class CFinPersistance : public CBasePersistant {

	public:
		CFinPersistance () : CBasePersistant (true, false) {};
		virtual ~CFinPersistance () {};

		CFinPersistance (const CFinPersistance& kprModele) = delete;
		CFinPersistance& operator= (const CFinPersistance& kprModele) = delete;

		virtual uint64_t taille () const { return 0; };
		virtual void ecrire (std::ostream& prFlux) const {};
		virtual std::string nomClasse () const { return "persistance::CFinPersistance"; };

		static CBasePersistant* constructeur (std::istream& prFlux) { return nullptr; };

	protected:
		CFinPersistance (std::istream& prFlux) : CBasePersistant::CBasePersistant (prFlux) {};
	};

// ________________________________________________________________________________________
uint64_t CBasePersistant::smvCompteur = 0;
uint64_t CBasePersistant::smvNbNonAlloue = 0;
std::vector<CBasePersistant*>* CBasePersistant::smaLstObjets = nullptr;
std::map<std::string, CBasePersistant::constructeur>* CBasePersistant::smaReferentiel = nullptr;

// ________________________________________________________________________________________
CBasePersistant::CBasePersistant (bool pvPersistant, bool pvAlloue) :
	mvPersistant (pvPersistant),
	mvAlloue (pvAlloue) {

	if (!smaLstObjets)
		smaLstObjets = new std::vector<CBasePersistant*>;
	smaLstObjets->push_back (this);
	smvCompteur++;
	if (!mvAlloue)
		smvNbNonAlloue++;
	};

// ________________________________________________________________________________________
CBasePersistant::CBasePersistant (std::istream& prFlux) :
	mvPersistant (true),
	mvAlloue (true) {

	if (!smaLstObjets)
		smaLstObjets = new std::vector<CBasePersistant*>;
	smvCompteur++;
	smaLstObjets->push_back (this);
	};

// ________________________________________________________________________________________
CBasePersistant::~CBasePersistant () {

	smvCompteur--;
	if (!mvAlloue)
		smvNbNonAlloue--;

	std::vector<CBasePersistant*>::const_iterator liObjet=std::find (smaLstObjets->begin (), smaLstObjets->end (), this);
	smaLstObjets->erase (liObjet);

	if (!smvNbNonAlloue && smaLstObjets) {
		if (!smaLstObjets->empty ()) {
			CBasePersistant* laObjet = smaLstObjets->at (smaLstObjets->size ()-1);
			if (laObjet->mvAlloue)
				delete laObjet;
			};
		};

	if (smaLstObjets && smaLstObjets->empty ()) {
		delete smaLstObjets;
		smaLstObjets = nullptr;
		if (smaReferentiel) {
			delete smaReferentiel;
			smaReferentiel = nullptr;
			};
		};
	};

// ________________________________________________________________________________________
void CBasePersistant::toutEcrire (std::ostream& prFlux) {

	if (smaLstObjets) {
		for (std::vector<CBasePersistant*>::iterator liObjet = smaLstObjets->begin (); liObjet != smaLstObjets->end (); ++liObjet) {
			CBasePersistant* laObjet = *liObjet;
			if (laObjet->mvPersistant) {
				laObjet->ecrire (prFlux);
				};
			};
		};
	};

// ________________________________________________________________________________________
void CBasePersistant::referencer (const std::string& kprNomClasse, constructeur fnConstructeur) {

	if (!smaReferentiel)
		smaReferentiel = new std::map<std::string, CBasePersistant::constructeur>;
	smaReferentiel->insert (std::pair<const std::string, CBasePersistant::constructeur> (kprNomClasse, fnConstructeur));
	};

// Créer un objet persistant factice non alloue.
// Sa destruction entraînera obligatoirement l'appel au destructeur de CBasePersistant.
// La destruction de dernier objet persistants non alloué (il y aura au moins celui-ci) déclenchera la libération de tous les objets persistants alloués.
static CFinPersistance gvFin;

};
